
%case #2. BER VS N. Comparing theoretical, experimental and Gaussian
%approximation.

function Figure2_2

clear all
close all

flag =0; %set this 1 to get experimental values
beta = 50;
s = 40;
r0 = 1.1;
N0 = 50/10^0.2;    %E_b/N_0= 2dB, E_b=2*beta*E[x^2]=beta
b = sqrt(2)/2;

%Integration upper limit
maxx = int_up_lim(b);

  
%get the theoritical values(Average)to y1-y3 and get the gaussian
%approximated values to g1-g3
 vec_N=4:4:24;
 k=1;
 for j=1:length(vec_N),
    N =vec_N(j);
    y1(k) = quad(@(x)intfun(x,N,beta,s,1,r0,N0,b),0,maxx);
    y2(k) = quad(@(x)intfun(x,N,beta,s,20,r0,N0,b),0,maxx);
    y3(k) = quad(@(x)intfun(x,N,beta,s,39,r0,N0,b),0,maxx);
 
    g1(k) = gauss_approx(N,beta,s,1,N0,b,r0);
    g2(k) = gauss_approx(N,beta,s,20,N0,b,r0);
    g3(k) = gauss_approx(N,beta,s,39,N0,b,r0);
 
    k=k+1;
 
 end
 
 switch flag
     case 1
        [y4,y5,y6]=experimental;    %call the function which calculates BER based on simulation
     case 0
        load('y4.mat'); % simulated BER for tau=1
        load('y5.mat'); % simulated BER for tau=20
        load('y6.mat'); % simulated BER for tau=39
 end
 
 
 figure
 plot(vec_N,y1,'-ko',vec_N,y2,'-ro',vec_N,y3,'-bo',vec_N,y4,':k*',vec_N,y5,':r*',vec_N,y6,':b*',vec_N,g1,'^k--',vec_N,g2,'^r--',vec_N,g3,'^b--'),axis([4,24,0,0.22])
 title('BER Vs N for various delays \tau')
 xlabel('N')
 ylabel('BER')
 legend('\tau = 1 (th)','\tau = 20 (th)', '\tau =39 (th)', '\tau = 1 (exp)','\tau = 20 (exp)', '\tau =39 (exp)','\tau = 1 (app)','\tau = 20 (app)', '\tau =39 (app)','Location','NorthEastOutside')
 saveas(gcf,'BER_Vs_N_Figure2_2','epsc')
end


function maxx = int_up_lim(b)

MAX_I = 10^6; 
step = 10^(-2);
MAX_CDF = 0.99999;

maxx = b;
no_it = 1;
while no_it<MAX_I,
     maxx = maxx+step;
     if( quad(@(x)intfun1(x,b),0,maxx)>=MAX_CDF ),
        break;   
     else
        no_it=no_it+1;
    end    
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = intfun1(x,b)

y = (x./b^2).*exp((-x.^2)./(2*b^2));

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = intfun(x,N,beta,s,tau,r0,N0,b)

D1 = (s/tau)*(r0+1)-1;
D2 = 1+(1/r0)*(1-tau/s);
T1 =  N/beta/D1^2 + (beta-1)/(beta^2)/D1 + (2*N-1)/(2*beta);
T2 = (2*N0)./(x.^2*(beta*D2^2));
xi = T1+T2;
y1 = erfc(xi.^(-1/2))/2;
y2 = (x./b^2).*exp((-x.^2)./(2*b^2));
y = y1.*y2;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Calculate GAUSSIAN approximated values
function ABER = gauss_approx(N,beta,s,tau,N0,b,r0)

%Compute E[z^2]
Ealz_2 = 2*b^2;

term = s+(s-tau)/r0;

EA_2 = (1/4)*N*(2*beta) + (1/4)*(2*beta)^2 - (1/8)*(2*beta);
EB_2 = (1/4)*N*(2*beta);
EAB = (1/8)*(2*beta-2);
EC_2 = N0*beta;

T1 = Ealz_2*(term^2)*EA_2;
T2 = Ealz_2*(tau/r0)^2*EB_2;
T3 = s^2*EC_2;
T4 = 2*Ealz_2*term*(tau/r0)*EAB;

Ez_2 = T1 + T2 + T3 + T4;

%Compute E[z]
Ez = (pi/2)^(1/2)*b*term*beta;

%Compute zeta
zeta = 2*Ez_2/Ez^2 - 2;

%Compute approx. BER
ABER = erfc(zeta^(-1/2))/2;

end

