
%Case4, BER Vs SNR for lambda =1/5 and lambda=1/20

function Figure2_10

close all
clear all

%Parameters for Rayleigh distributions
b = sqrt(2)/2;
bt = 0.9*b;

%Integration upper limit
[maxx maxy] = int_up_lim(b,bt);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parameter for the Exponential distribution
vec_lambda = [1/5 1/20];
for i=1:length(vec_lambda),
    lambda = vec_lambda(i);
    ell = exp(-lambda);
    draw_fig(i,lambda,ell,maxx,maxy,b,bt);
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function draw_fig(fig,lambda,ell,maxx,maxy,b,bt)
flag=0;
N = 4;
beta = 50;
s = 40;
Pr = (1-ell)/(1-ell^s)*(ell.^(0:s-1));
vec_tau = 0:s-1;
vec_SNR = 1:8;
vec_N0 = beta./(10.^(vec_SNR/10));

MatA_tau = zeros(1,length(vec_N0));
MatA_tauc = zeros(1,length(vec_N0));
Mat = zeros(1,length(vec_N0));

for j=1:length(vec_N0),
    N0 = vec_N0(j);
    %Approx tauc - Formulae in (1.35)-(1.36)
    Etau = 1/lambda;
    Etau_2 = 2/lambda^2;
    MatA_tauc(j) = gauss_approx(N,beta,s,N0,b,bt,Etau,Etau_2);
    %Approx tau - Formulae in (1.33)-(1.34)
    Etau = (ell/(1-ell^s))*((s-1)*ell^s-s*ell^(s-1)+1)/(1-ell);
    Etau_2 = Etau + (ell^2/(1-ell^s))*(2*(1-ell^s)/(1-ell)^2-2*ell^(s-1)*s/(1-ell)-ell^(s-2)*s*(s-1));
    MatA_tau(j) = gauss_approx(N,beta,s,N0,b,bt,Etau,Etau_2);
    temp = zeros(s,1);
    for i=1:length(vec_tau),
        tau = vec_tau(i);
        temp(i) = quad2d(@(x,y)intfun(x,y,N,beta,s,tau,N0,b,bt),0,maxx,0,maxy);
    end
    Mat(j) = Pr*temp;
end

switch flag
     case 1
        experimental1;%call the function which calculates BER based on simulation
        experimental2;
     case 0
        load('BER1');
        load('BER2');
        
end
  
figure(1);
subplot(1,2,fig);

if fig==1,
    plot(vec_SNR,Mat,'k-', ...
    vec_SNR,MatA_tauc,'k*-', ...
    vec_SNR,MatA_tau,'ko-',...
    vec_SNR,BER1,'r*-'),axis([1 8 0 0.1]);
    title('\lambda=1/5');
    FigHandle = fig;
    set(FigHandle, 'Position', [5, 5, 1900, 850]);
    h_legend=legend('Th','Gauss app(c)','Gauss app','exp','location','NorthEast');
    set(h_legend,'FontSize',8);

else
    plot(vec_SNR,Mat,'k-', ...
    vec_SNR,MatA_tauc,'k*-', ...
    vec_SNR,MatA_tau,'ko-',...
    vec_SNR,BER2,'r*-'),axis([1 8 0 0.14]);
    title('\lambda=1/20');

end

xlabel('E_b/N_0(dB)');
ylabel('BER');
saveas(gcf,'BER_Vs_SNR_Figure2_10','epsc')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ABER = gauss_approx(N,beta,s,N0,b,bt,Etau,Etau_2)

Ealz = b*(pi/2)^(1/2);
Ealz_2 = 2*b^2;
Ealo = bt*(pi/2)^(1/2);
Ealo_2 = 2*bt^2;

EA = (1/2)*(2*beta);
EA_2 = (1/4)*N*(2*beta) + (1/4)*(2*beta)^2 - (1/8)*(2*beta);
EB_2 = (1/4)*N*(2*beta);
EAB = (1/8)*(2*beta-2);
EC_2 = (N0/2)*(2*beta);

%Compute E[z]
Ed0 = -Etau*Ealo + s*(Ealz+Ealo);
Ez = Ed0*EA;

%Compute E[z^2]
Ed0_2 = Etau_2*Ealo_2 -2*s*Etau*(Ealo_2+Ealo*Ealz) +s^2*(Ealz_2+Ealo_2+2*Ealz*Ealo);
Ed1_2 = Etau_2*Ealo_2;
Ed1d2 = -Etau_2*Ealo_2 +Etau*s*(Ealz*Ealo+Ealo_2);

Ez_2 = Ed0_2*EA_2 +Ed1_2*EB_2 +s^2*EC_2 +2*Ed1d2*EAB; 

%Compute zeta
zeta = 2*Ez_2/Ez^2 - 2;

%Compute approx. BER
ABER = erfc(zeta^(-1/2))/2;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [maxx maxy] = int_up_lim(b,bt)

MAX_I = 10^6; 
step = 10^(-2);
MAX_CDF = 0.99999;

maxx = b;
maxy = bt;
no_it = 1;
while no_it<MAX_I,
     maxx = maxx+step;
     maxy = maxy+step;
     if( quad2d(@(x,y)intfun1(x,y,b,bt),0,maxx,0,maxy)>=MAX_CDF ),
        break;   
     else
        no_it=no_it+1;
    end    
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function z = intfun1(x,y,b,bt)

z1 = (x./b^2).*exp((-x.^2)./(2*b^2));
z2 = (y./bt^2).*exp((-y.^2)./(2*bt^2));
z = z1.*z2;

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function z = intfun(x,y,N,beta,s,tau,N0,b,bt)

delta0 = x*s + y*(s-tau);
delta1 = y*tau;
rat = delta1./delta0;
T1 = (rat.^2)*(N/beta) + rat*((beta-1)/beta^2) + (2*N-1)/2/beta;
T2 = (1./delta0).^2*N0*(2*s^2/beta);
xi = T1+T2;
z1 = erfc(xi.^(-1/2))/2;
z2 = (x./b^2).*exp((-x.^2)./(2*b^2));
z3 = (y./bt^2).*exp((-y.^2)./(2*bt^2));
z = (z1.*z2).*z3;

end







