function Generate_Fig_BER_LB

close all
clear all

%Parameter for Rayleigh distributions
b = sqrt(2)/2;

%Integration upper limit
maxx = int_up_lim(b);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIGURE 1
%Parameters CDMA system
N = 4;
beta = 50;
s = 40;
vec_tau = [1 20 39];
vec_SNR = 1:16;
vec_N0 = beta./(10.^(vec_SNR/10));
r0 = 1.1;

%Matrix results - average BER
Mat = zeros(length(vec_tau),length(vec_N0));
%Matrix results - lower bound
MatLB = Mat;

for i=1:length(vec_tau),
    tau = vec_tau(i);
    for j=1:length(vec_N0),
        N0 = vec_N0(j);
        Mat(i,j) = quad(@(x)intfun(x,N,beta,s,tau,r0,N0,b),0,maxx);
        MatLB(i,j) = comp_lb(beta,s,tau,r0,N0,b);
    end
end

figure(1);
subplot(1,2,1);
semilogy(vec_SNR,Mat(1,:),'ko-', ...
    vec_SNR,MatLB(1,:),'k+:', ...
    vec_SNR,Mat(2,:),'ro-', ...
    vec_SNR,MatLB(2,:),'r+:', ...
    vec_SNR,Mat(3,:),'bo-', ...
    vec_SNR,MatLB(3,:),'b+:');
legend('\tau=1(th)', ...
    '\tau=1(LB)', ...
    '\tau=20(th)', ...
    '\tau=20(LB)', ...
    '\tau=39(th)', ...
    '\tau=39(LB)', ...
    'Location', 'Best');
xlabel('E_b/N_0(dB)');
ylabel('BER(th) and LB');
title('N=4');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FIGURE 2
%Parameters CDMA system
N = 40;
beta = 50;
s = 40;
vec_tau = [1 20 39];
vec_SNR = 1:16;
vec_N0 = beta./(10.^(vec_SNR/10));
r0 = 1.1;

%Matrix results - average BER
Mat = zeros(length(vec_tau),length(vec_N0));
%Matrix results - lower bound
MatLB = Mat;

for i=1:length(vec_tau),
    tau = vec_tau(i);
    for j=1:length(vec_N0),
        N0 = vec_N0(j);
        Mat(i,j) = quad(@(x)intfun(x,N,beta,s,tau,r0,N0,b),0,maxx);
        MatLB(i,j) = comp_lb(beta,s,tau,r0,N0,b);
    end
end

figure(1);
subplot(1,2,2);
semilogy(vec_SNR,Mat(1,:),'ko-', ...
    vec_SNR,MatLB(1,:),'k+:', ...
    vec_SNR,Mat(2,:),'ro-', ...
    vec_SNR,MatLB(2,:),'r+:', ...
    vec_SNR,Mat(3,:),'bo-', ...
    vec_SNR,MatLB(3,:),'b+:');
% legend('\tau=1(th)', ...
%     '\tau=1(LB)', ...
%     '\tau=20(th)', ...
%     '\tau=20(LB)', ...
%     '\tau=39(th)', ...
%     '\tau=39(LB)', ...
%     'Location', 'SouthWest');
xlabel('E_b/N_0(dB)');
ylabel('BER(th) and LB');
title('N=40');

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function maxx = int_up_lim(b)

MAX_I = 10^6; 
step = 10^(-2);
MAX_CDF = 0.99999;

maxx = b;
no_it = 1;
while no_it<MAX_I,
     maxx = maxx+step;
     if( quad(@(x)intfun1(x,b),0,maxx)>=MAX_CDF ),
        break;   
     else
        no_it=no_it+1;
    end    
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = intfun1(x,b)

y = (x./b^2).*exp((-x.^2)./(2*b^2));

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = intfun(x,N,beta,s,tau,r0,N0,b)

D1 = (s/tau)*(r0+1)-1;
D2 = 1+(1/r0)*(1-tau/s);
T1 =  N/beta/D1^2 + (beta-1)/(beta^2)/D1 + (2*N-1)/(2*beta);
T2 = (2*N0)./(x.^2*(beta*D2^2));
xi = T1+T2;
y1 = erfc(xi.^(-1/2))/2;
y2 = (x./b^2).*exp((-x.^2)./(2*b^2));
y = y1.*y2;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function lb = comp_lb(beta,s,tau,r0,N0,b)
%Output:
%lb = lower-bound given by the formula in (1.22)

D2 = 1+(1/r0)*(1-tau/s);
w = (2*N0)/(beta*D2^2);
barg = 2*b^2;
lb = (1/2)*(1-1/sqrt(1+w/barg));

end





